'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getInit = exports.initIndex = exports.initDestroy = exports.initShow = exports.initEdit = exports.initUpdate = exports.initNew = exports.initCreate = undefined;

var _constants = require('./constants');

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

// Define initial states.
var initCreate = {};
var initNew = {};
var initUpdate = {};
var initEdit = {};
var initShow = {
  headers: {}
};
var initDestroy = {};
var initIndex = {
  pagination: {
    total: 0,
    perPage: 10,
    page: 1
  }
};

// Group actions by resource granulation.
var SINGLE_RESOURCE = [initCreate, initNew, initUpdate, initEdit, initShow, initDestroy];
var PLURAL_RESOURCES = [initIndex];

// Completes keys on states by granularity.
SINGLE_RESOURCE.forEach(function (init) {
  init.record = null;
  init.fetching = false;
  init.error = null;
});
PLURAL_RESOURCES.forEach(function (init) {
  init.records = null;
  init.fetching = false;
  init.error = null;
});

/**
 * Get the initial state for given action
 *
 * @param {string} action - The action requested.
 */
var getInit = function getInit(action) {
  var _initialStates;

  var initialStates = (_initialStates = {}, _defineProperty(_initialStates, _constants.BACKEND_ACTIONS.CREATE, initCreate), _defineProperty(_initialStates, _constants.BACKEND_ACTIONS.NEW, initNew), _defineProperty(_initialStates, _constants.BACKEND_ACTIONS.UPDATE, initUpdate), _defineProperty(_initialStates, _constants.BACKEND_ACTIONS.EDIT, initEdit), _defineProperty(_initialStates, _constants.BACKEND_ACTIONS.SHOW, initShow), _defineProperty(_initialStates, _constants.BACKEND_ACTIONS.DESTROY, initDestroy), _defineProperty(_initialStates, _constants.BACKEND_ACTIONS.INDEX, initIndex), _initialStates);

  return initialStates[action];
};

exports.initCreate = initCreate;
exports.initNew = initNew;
exports.initUpdate = initUpdate;
exports.initEdit = initEdit;
exports.initShow = initShow;
exports.initDestroy = initDestroy;
exports.initIndex = initIndex;
exports.getInit = getInit;