'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _reduxActions = require('redux-actions');

var _constants = require('./constants');

var _init = require('./init');

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var init = {};

/**
 * Generic reducer for start actions.
 */
var apiStart = function apiStart(key) {
  return function (state, action) {
    var _extends2;

    return _extends({}, (0, _init.getInit)(action.meta.apiAction), (_extends2 = {}, _defineProperty(_extends2, key, state[key]), _defineProperty(_extends2, 'fetching', true), _extends2));
  };
};

/**
 * Generic reducer for start failure actions.
 */
var apiFailure = function apiFailure(key) {
  return function (state, action) {
    var _extends3;

    var meta = action.meta,
        payload = action.payload;


    return _extends({}, (0, _init.getInit)(meta.apiAction), (_extends3 = {}, _defineProperty(_extends3, key, state[key]), _defineProperty(_extends3, 'error', {
      code: meta.status.code,
      messages: payload.response
    }), _extends3));
  };
};

/**
 * Generic reducer for success actions.
 */
var apiSuccess = function apiSuccess(_state, action) {
  return _extends({}, (0, _init.getInit)(action.meta.apiAction), {
    record: action.payload
  });
};

/**
 * Reducer for success index action.
 */
var apiIndexSuccess = function apiIndexSuccess(state, action) {
  var payload = action.payload,
      meta = action.meta;

  // Record list.

  var records = !meta.additive ? payload.records : [].concat(_toConsumableArray(state.records || []), _toConsumableArray(payload.records));

  return _extends({}, (0, _init.getInit)(meta.apiAction), {
    pagination: payload.pagination,
    records: records
  });
};

/**
 * Reducer for success show action.
 */
var apiShowSuccess = function apiShowSuccess(_state, action, apiAction) {
  var payload = action.payload;


  return _extends({}, (0, _init.getInit)(apiAction), {
    record: payload.record,
    headers: payload.headers
  });
};

/**
 * Generic reducer for to clear action.
 */
var clear = function clear(_state, action) {
  return (0, _init.getInit)(action.payload.apiAction);
};

var handleApiActions = function handleApiActions() {
  var _handleActions;

  return (0, _reduxActions.handleActions)((_handleActions = {}, _defineProperty(_handleActions, _constants.ACTIONS.API_CREATE_START, apiStart('record')), _defineProperty(_handleActions, _constants.ACTIONS.API_CREATE_SUCCESS, apiSuccess), _defineProperty(_handleActions, _constants.ACTIONS.API_CREATE_FAILURE, apiFailure('record')), _defineProperty(_handleActions, _constants.ACTIONS.API_NEW_START, apiStart('record')), _defineProperty(_handleActions, _constants.ACTIONS.API_NEW_SUCCESS, apiSuccess), _defineProperty(_handleActions, _constants.ACTIONS.API_NEW_FAILURE, apiFailure('record')), _defineProperty(_handleActions, _constants.ACTIONS.API_UPDATE_START, apiStart('record')), _defineProperty(_handleActions, _constants.ACTIONS.API_UPDATE_SUCCESS, apiSuccess), _defineProperty(_handleActions, _constants.ACTIONS.API_UPDATE_FAILURE, apiFailure('record')), _defineProperty(_handleActions, _constants.ACTIONS.API_INDEX_START, apiStart('records')), _defineProperty(_handleActions, _constants.ACTIONS.API_INDEX_SUCCESS, apiIndexSuccess), _defineProperty(_handleActions, _constants.ACTIONS.API_INDEX_FAILURE, apiFailure('records')), _defineProperty(_handleActions, _constants.ACTIONS.API_SHOW_START, apiStart('record')), _defineProperty(_handleActions, _constants.ACTIONS.API_SHOW_SUCCESS, apiShowSuccess), _defineProperty(_handleActions, _constants.ACTIONS.API_SHOW_FAILURE, apiFailure('record')), _defineProperty(_handleActions, _constants.ACTIONS.API_EDIT_START, apiStart('record')), _defineProperty(_handleActions, _constants.ACTIONS.API_EDIT_SUCCESS, apiSuccess), _defineProperty(_handleActions, _constants.ACTIONS.API_EDIT_FAILURE, apiFailure('record')), _defineProperty(_handleActions, _constants.ACTIONS.API_DESTROY_START, apiStart('record')), _defineProperty(_handleActions, _constants.ACTIONS.API_DESTROY_SUCCESS, apiSuccess), _defineProperty(_handleActions, _constants.ACTIONS.API_DESTROY_FAILURE, apiFailure('record')), _defineProperty(_handleActions, _constants.ACTIONS.CLEAR, clear), _handleActions), {});
};

// eslint-disable-next-line no-useless-escape
var regexp = new RegExp('^' + _constants.NAME + '.API_\\w+$');

var handleAction = function handleAction(actionType) {
  return regexp.test(actionType) || actionType === _constants.ACTIONS.CLEAR;
};

/**
 * Reducer adapter to api actions.
 */
var apiReducerAdapter = function apiReducerAdapter() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : init;
  var action = arguments[1];

  if (!handleAction(action.type)) return state;

  var source = regexp.test(action.type) ? action.meta : action.payload;
  var apiAction = source.apiAction,
      apiResource = source.apiResource;

  // Get Api Redux state of resource.

  var resourceState = state[apiResource] || {};

  // Get State of Api action about resource.
  // If not defined then get initial state of action.
  var apiActionState = resourceState[apiAction] || (0, _init.getInit)(apiAction);

  return _extends({}, state, _defineProperty({}, apiResource, _extends({}, resourceState, _defineProperty({}, apiAction, handleApiActions()(apiActionState, action)))));
};

exports.default = apiReducerAdapter;